# -*- coding: utf-8 -*-
# flake8: noqa

'''
top-level package for pynlai.
'''


from .interface import (
    Trigger,
    Argument,
    nl_function,
    run,
)


__author__ = '''Chris Pappalardo'''
__email__ = 'cpappala@yahoo.com'
__version__ = '0.1.2'
